package com.regnstad.csv2jsonosx;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.fife.ui.rsyntaxtextarea.RSyntaxTextArea;
import org.fife.ui.rsyntaxtextarea.SyntaxConstants;
import org.fife.ui.rtextarea.RTextScrollPane;
import org.json.me.JSONArray;
import org.json.me.JSONException;
import com.regnstad.csv2jsonosx.parser.ParseHandler;
import com.regnstad.csv2jsonosx.parser.ParseHandler.ParseObserver;

public class CSV2JSONOSX extends JFrame implements KeyListener, ParseObserver {

	private static final long serialVersionUID = -7983551355663100310L;
	final RSyntaxTextArea textArea = new RSyntaxTextArea(40, 160);
	private static final Clipboard CLIPBOARD = Toolkit.getDefaultToolkit().getSystemClipboard(); 
	
	public static boolean useHeadings = true;

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new CSV2JSONOSX().setVisible(true);
			}
		});

	}

	public CSV2JSONOSX(){
		JPanel cp = new JPanel(new BorderLayout());
		textArea.addKeyListener(this);

		textArea.setBackground(new Color(0xefefef));
		textArea.setCaretColor(new Color(0x00ffcc));
		textArea.setSyntaxEditingStyle(SyntaxConstants.SYNTAX_STYLE_JAVASCRIPT);
		textArea.setCodeFoldingEnabled(true);
		textArea.setAntiAliasingEnabled(true);
		RTextScrollPane sp = new RTextScrollPane(textArea);
		sp.setFoldIndicatorEnabled(true);
		cp.add(sp);

		setContentPane(cp);
		setTitle("CSV > JSON");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		pack();
		setLocationRelativeTo(null);	
	}
	
	@Override
	public void keyPressed(KeyEvent arg0) {
	}

	@Override
	public void keyReleased(KeyEvent event) {
		if (event.isMetaDown()) { 
			if (event.getKeyCode()==KeyEvent.VK_V) {
				String pasteString = ""; 
				try { 
					pasteString = (String)(CLIPBOARD.getContents(this).getTransferData(DataFlavor.stringFlavor)); 
					Thread parseCSVThread = new Thread(new ParseHandler(CSV2JSONOSX.this, pasteString, useHeadings));
                    parseCSVThread.start();					
				} catch (Exception e) {
					log(e.toString());
				}          
			}
		} 
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
	}

	@Override
	public void handleParsedJSON(JSONArray json) {
		try {
			log("handleParsedJSON: " + json.toString(5));
			textArea.setText(json.toString(5));
		} catch (JSONException e) {
			log("handleParsedJSON" + e.toString());
		}
		
	}

	@Override
	public void handleJSONParseError(String message) {
		log("handleJSONParseError: " + message);
		
	}
	
	private void log(String message){
		System.out.println("JSONDrop: " + message);
	}

}
