package com.regnstad.csv2jsonosx.parser;

import au.com.bytecode.opencsv.CSVParser;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

/**
 *
 * @author primer
 */
public class CSV2JSONParser {

    public boolean useHeadings = true;
    private ParsingObserver observer;
    private String csv;

    public CSV2JSONParser(ParsingObserver observer, String csv) {
        this.observer = observer;
        this.csv = csv;
    }

    public void setUseHeadings(boolean useHeadings) {
        this.useHeadings = useHeadings;
    }

    public void convert() {
        JSONArray jsonArray = new JSONArray();
        CSVParser parser = new CSVParser();
        String[] headers = null;
        String str;

        BufferedReader reader = new BufferedReader(new StringReader(csv));
        int lineNumber = 0;
        try {
            while ((str = reader.readLine()) != null) {
                lineNumber++;
                if (str.length() > 0) {
                    //TODO - this code doesn't handle nested quotations... we just strip out the inverted commas
                    //This can cause column count mismatches where quoted text includes a comma.
                    str = str.replace("\"", "");
                    if (lineNumber == 1 && useHeadings) {
                        headers = parser.parseLine(str);
                    } else {
                        if (useHeadings) {
                            if (headers != null) {
                                //System.out.println("- " + lineNumber + ": " + str);
                                String[] fields = parser.parseLine(str);
                                int numFields = fields.length;
                                if (numFields == headers.length) {
                                    //The JSON lib can't handle very large datasets - this method will only ever 
                                    //work on small to medium sized files.
                                    JSONObject json = new JSONObject();
                                    for (int i = 0; i < numFields; i++) {
                                        if (headers[i].length() > 0) { 
                                            json.put(headers[i], fields[i].trim());
                                        }
                                    }
                                    //We're getting an out of memory error here on large datasets...
                                    jsonArray.put(json);
                                    json = null;
                                } else {
                                    JSONObject error = new JSONObject();
                                    error.put("Error", "Expected " + headers.length + " fields but got " + numFields);
                                    jsonArray.put(error);
                                }
                            }
                        } else {
                            String[] fields = parser.parseLine(str);
                            int numFields = fields.length;
                            JSONArray json = new JSONArray();
                            for (int i = 0; i < numFields; i++) {
                                if (fields[i].length() > 0) {
                                    json.put(fields[i]);
                                }
                            }
                            jsonArray.put(json);
                        }
                    }
                }
            }
            //Output json to console:
            Logger.getAnonymousLogger().log(Level.INFO, "Returning JSON...");
            observer.handleParsingSuccess(jsonArray);
        } catch (JSONException ex) {
            Logger.getLogger(CSV2JSONParser.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException e) {
            Logger.getLogger(CSV2JSONParser.class.getName()).log(Level.SEVERE, null, e);
        }
    }

    public interface ParsingObserver {

        public void handleParsingSuccess(JSONArray jsonArray);

        public void handleParsingError(String message);
    }
}
